# encoding: utf-8
def underscore(camel_cased_word)
  camel_cased_word.to_s.gsub(/::/, '/').
    gsub(/([A-Z]+)([A-Z][a-z])/,'\1_\2').
    gsub(/([a-z\d])([A-Z])/,'\1_\2').
    tr("-", "_").
    downcase
end

module JobParser
  class ParseSchema < Parser

    EXTRA_SCHEMA_TEXT_FIELDS = %w{
      benefits educationRequirements incentives industry
      occupationalCategory qualifications responsibilities skills specialCommitments workHours
    }

    EXTRA_SCHEMA_TEXT_FIELDS.each do |field|
      define_method("job_#{underscore(field)}") {
        get_content_at_prop(field)
      }

      private "job_#{underscore(field)}".to_sym
    end

    def initialize(html, from_url)
      Parser::ACCEPTED_ELEMENTS.push("span")
      super(html, from_url)
    end

    def job
      res = super
      res[:schema] = true
      unless res[:from_cache]
        EXTRA_SCHEMA_TEXT_FIELDS.each do |field|
          underscore_name = underscore(field).to_sym
          result = send("job_#{underscore_name}")
          res[underscore_name] = result unless result.nil? || result.empty?
        end
      end
      cache(res)
    end

    private

    def job_salary
      salary = job_salary_string
      SALARY_GROUP_REGEX.match(salary.gsub(CLEAN_SALARY_REGEX, "")) { |match|
        [match[1].to_i, match[2].to_i]
      }
    end

    def job_title
      get_content_at_prop("title")
    end

    def apply_link
      Facets::Apply.new(@doc, @url, @plain_text).parse
    end

    def job_salary_string
      get_content_at_prop("baseSalary")
    end

    def job_location
      if @doc.css("*").any? { |elem| elem['itemtype'] == "http://schema.org/PostalAddress" }
        fields = %w{ streetAddress addressLocality addressRegion addressCountry postalCode }
        address = []
        fields.each do |field|
          content = get_content_at_prop(field)
          address.push(content) unless content.empty?
        end
        address.join(", ")
      else
        # some sites don't use the address stuff properly
        if is_content_at_prop?("addressLocality")
          get_content_at_prop("addressLocality")
        else
          get_content_at_prop("jobLocation")
        end
      end
    end

    def job_postcode
      POSTCODE_REGEX.match(job_location) { |m| return m[0] }
    end

    def deadline
    end

    def does_use_schema?
      @doc.css("*").any? { |elem|
        elem['itemtype'] == "http://schema.org/JobPosting"
      }
    end

    def get_content_at_prop(prop)
      elem = find_with_itemprop(prop)
      elem ? Cleaner.strip_all_white_space(elem.content) : ""
    end

    def is_content_at_prop?(prop)
      elem = find_with_itemprop(prop)
      elem && !elem.content.empty?
    end

    def find_with_itemprop(prop)
      @doc.css("*").select { |elem|
        elem['itemprop'] == prop
      }.first
    end
  end
end
